<?php

namespace Drupal\ip_geoloc\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\ip_geoloc\Services\IpGeoLocAPI;

/**
 * Pending doc.
 */
class IpLookupForm extends FormBase {

  protected $ipGeolocApi;

  /**
   * Create the form, adds dependency injection.
   */
  public function __construct(ConfigFactoryInterface $config_factory, IpGeoLocAPI $ipGeolocApi) {
    parent::__construct($config_factory);
    $this->ipGeolocApi = $ipGeolocApi;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ip_geoloc_ip_lookup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Migration comment:  Part of ip_geoloc_set_location_form definition.
    $config = \Drupal::config('ip_geoloc.settings');

    $last_visit = $form_state->has('last_visit') ? $form_state->get('last_visit') : '';
    $formatted_address = $form_state->has('formatted_address') ? $form_state->get('formatted_address') : '';
    if (!empty($last_visit) || !empty($formatted_address)) {
      $prefix = '<div class="ip_geoloc_address_lookup">' . $formatted_address . (empty($last_visit) ? '' : "<br/>$last_visit") . '</div>';
    }
    $form['ip_address'] = [
      '#type' => 'textfield',
      '#title' => t('IP address'),
      '#default_value' => '',
      '#size' => 16,
      '#required' => FALSE,
      '#prefix' => isset($prefix) ? $prefix : NULL,
    ];
    if (user_access('administer site configuration') && $config->get('ip_geoloc_store_addresses') ? $config->get('ip_geoloc_store_addresses') : TRUE) {
      $form['store'] = [
        '#type' => 'checkbox',
        '#title' => t('If found, store on IP geolocation database for future reference.'),
        '#default_value' => FALSE,
      ];
    }
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Lookup'),
      '#submit' => ['ip_geoloc_ip_lookup_submit'],
    ];
    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Migration comment:  Part of _ip_geoloc_process_go_to_submit for submiting ip_geoloc_set_location_form
    // $form_state->set(['storage', 'value'], 'something');.
    $store = !empty($form_state->getStorage());

    $location = $this->ipGeolocApi->getLocationByIp(trim($form_state->get('ip_address'), TRUE, $store));

    $connection = \Drupal::database();

    // @TODO Acesslog table doesn't exist for d8
    if ($connection->schema()->tableExists('accesslog')) {

      $last_visit = $connection->query('SELECT MAX(timestamp) FROM {accesslog} WHERE hostname = :ip_address', [':ip_address' => $location['ip_address']])->fetchField();
      if ($last_visit) {
        $last_visit = format_date($last_visit, 'short');

        $form_state->set(['storage', 'value'], 'last_visit');
        $form_state->set(
          ['storage',
            $this->t('<strong>Last visit:</strong> %date',
            ['%date' => $last_visit])
          ],
          'last_visit');
      }
    }
    $formatted_address = isset($location['formatted_address']) ? t('%ip: <strong>%address</strong>', ['%ip' => $location['ip_address'], '%address' => $location['formatted_address']]) : '';

    $form_state->set(['storage', $formatted_address], 'formatted_address');

    // To preserve entered values and storage array.
    $form_state->setRebuild();
  }

}
